@@ -3,7 +3,7 @@
    Support for executable statements. */
 
 /*
- * Copyright (c) 2009,2012 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2009,2013-2015 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1998-2003 by Internet Software Consortium
  *
@@ -25,12 +25,6 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
 #include "dhcpd.h"
@@ -39,7 +33,8 @@
 #include <sys/wait.h>
 
 int execute_statements (result, packet, lease, client_state,
-			in_options, out_options, scope, statements)
+			in_options, out_options, scope, statements,
+			on_star)
 	struct binding_value **result;
 	struct packet *packet;
 	struct lease *lease;
@@ -48,6 +43,7 @@ int execute_statements (result, packet, lease, client_state,
 	struct option_state *out_options;
 	struct binding_scope **scope;
 	struct executable_statement *statements;
+	struct on_star *on_star;
 {
 	struct executable_statement *r, *e, *next;
 	int rc;
@@ -59,14 +55,14 @@ int execute_statements (result, packet, lease, client_state,
 	if (!statements)
 		return 1;
 
-	r = (struct executable_statement *)0;
-	next = (struct executable_statement *)0;
-	e = (struct executable_statement *)0;
+	r = NULL;
+	next = NULL;
+	e = NULL;
 	executable_statement_reference (&r, statements, MDL);
 	while (r && !(result && *result)) {
-		if (r -> next)
-			executable_statement_reference (&next, r -> next, MDL);
-		switch (r -> op) {
+		if (r->next)
+			executable_statement_reference (&next, r->next, MDL);
+		switch (r->op) {
 		      case statements_statement:
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: statements");
@@ -74,7 +70,8 @@ int execute_statements (result, packet, lease, client_state,
 			status = execute_statements (result, packet, lease,
 						     client_state, in_options,
 						     out_options, scope,
-						     r -> data.statements);
+						     r->data.statements,
+						     on_star);
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: statements returns %d", status);
 #endif
@@ -83,42 +80,50 @@ int execute_statements (result, packet, lease, client_state,
 			break;
 
 		      case on_statement:
-			if (lease) {
-			    if (r -> data.on.evtypes & ON_EXPIRY) {
+			/*
+			 * if we haven't been passed an on_star block but
+			 * do have a lease, use the one from the lease
+			 * This handles the previous v4 calls.
+			 */
+			if ((on_star == NULL) && (lease != NULL))
+			    on_star = &lease->on_star;
+
+			if (on_star != NULL) {
+			    if (r->data.on.evtypes & ON_EXPIRY) {
 #if defined (DEBUG_EXPRESSIONS)
 				    log_debug ("exec: on expiry");
 #endif
-				if (lease -> on_expiry)
+				if (on_star->on_expiry)
 					executable_statement_dereference
-						(&lease -> on_expiry, MDL);
-				if (r -> data.on.statements)
+						(&on_star->on_expiry, MDL);
+				if (r->data.on.statements)
 					executable_statement_reference
-						(&lease -> on_expiry,
-						 r -> data.on.statements, MDL);
+						(&on_star->on_expiry,
+						 r->data.on.statements, MDL);
 			    }
-			    if (r -> data.on.evtypes & ON_RELEASE) {
+			    if (r->data.on.evtypes & ON_RELEASE) {
 #if defined (DEBUG_EXPRESSIONS)
 				    log_debug ("exec: on release");
 #endif
-				if (lease -> on_release)
+				if (on_star->on_release)
 					executable_statement_dereference
-						(&lease -> on_release, MDL);
-				if (r -> data.on.statements)
+						(&on_star->on_release, MDL);
+				if (r->data.on.statements)
 					executable_statement_reference
-						(&lease -> on_release,
-						 r -> data.on.statements, MDL);
+						(&on_star->on_release,
+						 r->data.on.statements, MDL);
 			    }
-			    if (r -> data.on.evtypes & ON_COMMIT) {
+			    if (r->data.on.evtypes & ON_COMMIT) {
 #if defined (DEBUG_EXPRESSIONS)
 				    log_debug ("exec: on commit");
 #endif
-				if (lease -> on_commit)
+				if (on_star->on_commit)
 					executable_statement_dereference
-						(&lease -> on_commit, MDL);
-				if (r -> data.on.statements)
+						(&on_star->on_commit, MDL);
+				if (r->data.on.statements)
 					executable_statement_reference
-						(&lease -> on_commit,
-						 r -> data.on.statements, MDL);
+						(&on_star->on_commit,
+						 r->data.on.statements, MDL);
 			    }
 			}
 			break;
@@ -130,15 +135,16 @@ int execute_statements (result, packet, lease, client_state,
 			status = (find_matching_case
 				  (&e, packet, lease, client_state,
 				   in_options, out_options, scope,
-				   r -> data.s_switch.expr,
-				   r -> data.s_switch.statements));
+				   r->data.s_switch.expr,
+				   r->data.s_switch.statements));
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: switch: case %lx", (unsigned long)e);
 #endif
 			if (status) {
 				if (!(execute_statements
 				      (result, packet, lease, client_state,
-				       in_options, out_options, scope, e))) {
+				       in_options, out_options, scope, e,
+				       on_star))) {
 					executable_statement_dereference
 						(&e, MDL);
 					return 0;
@@ -156,7 +162,7 @@ int execute_statements (result, packet, lease, client_state,
 			status = (evaluate_boolean_expression
 				  (&rc, packet,
 				   lease, client_state, in_options,
-				   out_options, scope, r -> data.ie.expr));
+				   out_options, scope, r->data.ie.expr));
 			
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: if %s", (status
@@ -169,18 +175,20 @@ int execute_statements (result, packet, lease, client_state,
 			if (!execute_statements
 			    (result, packet, lease, client_state,
 			     in_options, out_options, scope,
-			     rc ? r -> data.ie.tc : r -> data.ie.fc))
+			     rc ? r->data.ie.tc : r->data.ie.fc,
+			     on_star))
 				return 0;
 			break;
 
 		      case eval_statement:
 			status = evaluate_expression
-				((struct binding_value **)0,
-				 packet, lease, client_state, in_options,
-				 out_options, scope, r -> data.eval, MDL);
+				(NULL, packet, lease, client_state, in_options,
+				 out_options, scope, r->data.eval, MDL);
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: evaluate: %s",
 				   (status ? "succeeded" : "failed"));
+#else
+			POST(status);
 #endif
 			break;
 
@@ -272,16 +280,18 @@ int execute_statements (result, packet, lease, client_state,
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: return: %s",
 				   (status ? "succeeded" : "failed"));
+#else
+			POST(status);
 #endif
 			break;
 
 		      case add_statement:
 #if defined (DEBUG_EXPRESSIONS)
-			log_debug ("exec: add %s", (r -> data.add -> name
-					       ? r -> data.add -> name
+			log_debug ("exec: add %s", (r->data.add->name
+					       ? r->data.add->name
 					       : "<unnamed class>"));
 #endif
-			classify (packet, r -> data.add);
+			classify (packet, r->data.add);
 			break;
 
 		      case break_statement:
@@ -294,56 +304,57 @@ int execute_statements (result, packet, lease, client_state,
 		      case send_option_statement:
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: %s option %s.%s",
-			      (r -> op == supersede_option_statement
+			      (r->op == supersede_option_statement
 			       ? "supersede" : "send"),
-			      r -> data.option -> option -> universe -> name,
-			      r -> data.option -> option -> name);
+			      r->data.option->option->universe->name,
+			      r->data.option->option->name);
 			goto option_statement;
 #endif
 		      case default_option_statement:
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: default option %s.%s",
-			      r -> data.option -> option -> universe -> name,
-			      r -> data.option -> option -> name);
+			      r->data.option->option->universe->name,
+			      r->data.option->option->name);
 			goto option_statement;
 #endif
 		      case append_option_statement:
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: append option %s.%s",
-			      r -> data.option -> option -> universe -> name,
-			      r -> data.option -> option -> name);
+			      r->data.option->option->universe->name,
+			      r->data.option->option->name);
 			goto option_statement;
 #endif
 		      case prepend_option_statement:
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: prepend option %s.%s",
-			      r -> data.option -> option -> universe -> name,
-			      r -> data.option -> option -> name);
+			      r->data.option->option->universe->name,
+			      r->data.option->option->name);
 		      option_statement:
 #endif
-			set_option (r -> data.option -> option -> universe,
-				    out_options, r -> data.option, r -> op);
+			set_option (r->data.option->option->universe,
+				    out_options, r->data.option, r->op);
 			break;
 
 		      case set_statement:
 		      case define_statement:
+			status = 1;
 			if (!scope) {
 				log_error("set %s: no scope",
 					   r->data.set.name);
-				status = 0;
 				break;
 			}
 			if (!*scope) {
 			    if (!binding_scope_allocate(scope, MDL)) {
 				log_error("set %s: can't allocate scope",
 					  r->data.set.name);
-				status = 0;
 				break;
 			    }
 			}
 			binding = find_binding(*scope, r->data.set.name);
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug("exec: set %s", r->data.set.name);
+#else
+			POST(status);
 #endif
 			if (binding == NULL) {
 				binding = dmalloc(sizeof(*binding), MDL);
@@ -394,54 +405,57 @@ int execute_statements (result, packet, lease, client_state,
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: set %s%s", r -> data.set.name,
 				   (binding && status ? "" : " (failed)"));
+#else
+			POST(status);
 #endif
 			break;
 
 		      case unset_statement:
-			if (!scope || !*scope) {
-				status = 0;
+			if (!scope || !*scope)
 				break;
-			}
-			binding = find_binding (*scope, r -> data.unset);
+			binding = find_binding (*scope, r->data.unset);
 			if (binding) {
-				if (binding -> value)
+				if (binding->value)
 					binding_value_dereference
-						(&binding -> value, MDL);
+						(&binding->value, MDL);
 				status = 1;
 			} else
 				status = 0;
 #if defined (DEBUG_EXPRESSIONS)
-			log_debug ("exec: unset %s: %s", r -> data.unset,
+			log_debug ("exec: unset %s: %s", r->data.unset,
 				   (status ? "found" : "not found"));
+#else
+			POST(status);
 #endif
 			break;
 
 		      case let_statement:
 #if defined (DEBUG_EXPRESSIONS)
-			log_debug ("exec: let %s", r -> data.let.name);
+			log_debug("exec: let %s", r->data.let.name);
 #endif
-			ns = (struct binding_scope *)0;
+			status = 0;
+			ns = NULL;
 			binding_scope_allocate (&ns, MDL);
 			e = r;
 
 		      next_let:
 			if (ns) {
-				binding = dmalloc (sizeof *binding, MDL);
-				memset (binding, 0, sizeof *binding);
+				binding = dmalloc(sizeof(*binding), MDL);
+				memset(binding, 0, sizeof(*binding));
 				if (!binding) {
 				   blb:
-				    binding_scope_dereference (&ns, MDL);
+				    binding_scope_dereference(&ns, MDL);
 				} else {
-				    binding -> name =
-					    dmalloc (strlen
-						     (e -> data.let.name + 1),
-						     MDL);
-				    if (binding -> name)
-					strcpy (binding -> name,
-						e -> data.let.name);
+				    binding->name =
+					    dmalloc(strlen
+						    (e->data.let.name + 1),
+						    MDL);
+				    if (binding->name)
+					strcpy(binding->name,
+					       e->data.let.name);
 				    else {
-					dfree (binding, MDL);
-					binding = (struct binding *)0;
+					dfree(binding, MDL);
+					binding = NULL;
 					goto blb;
 				    }
 				}
@@ -450,35 +464,36 @@ int execute_statements (result, packet, lease, client_state,
 
 			if (ns && binding) {
 				status = (evaluate_expression
-					  (&binding -> value, packet, lease,
+					  (&binding->value, packet, lease,
 					   client_state,
 					   in_options, out_options,
-					   scope, e -> data.set.expr, MDL));
-				binding -> next = ns -> bindings;
-				ns -> bindings = binding;
+					   scope, e->data.set.expr, MDL));
+				binding->next = ns->bindings;
+				ns->bindings = binding;
 			}
 
 #if defined (DEBUG_EXPRESSIONS)
-			log_debug ("exec: let %s%s", e -> data.let.name,
-				   (binding && status ? "" : "failed"));
+			log_debug("exec: let %s%s", e->data.let.name,
+				  (binding && status ? "" : "failed"));
+#else
+			POST(status);
 #endif
-			if (!e -> data.let.statements) {
-			} else if (e -> data.let.statements -> op ==
+			if (!e->data.let.statements) {
+			} else if (e->data.let.statements->op ==
 				   let_statement) {
-				e = e -> data.let.statements;
+				e = e->data.let.statements;
 				goto next_let;
 			} else if (ns) {
 				if (scope && *scope)
-				    	binding_scope_reference (&ns -> outer,
-								 *scope, MDL);
+				    	binding_scope_reference(&ns->outer,
+								*scope, MDL);
 				execute_statements
-				      (result, packet, lease,
-				       client_state,
+				      (result, packet, lease, client_state,
 				       in_options, out_options,
-				       &ns, e -> data.let.statements);
+				       &ns, e->data.let.statements, on_star);
 			}
 			if (ns)
-				binding_scope_dereference (&ns, MDL);
+				binding_scope_dereference(&ns, MDL);
 			break;
 
 		      case log_statement:
@@ -486,15 +501,14 @@ int execute_statements (result, packet, lease, client_state,
 			status = (evaluate_data_expression
 				  (&ds, packet,
 				   lease, client_state, in_options,
-				   out_options, scope, r -> data.log.expr,
-				   MDL));
+				   out_options, scope, r->data.log.expr, MDL));
 			
 #if defined (DEBUG_EXPRESSIONS)
 			log_debug ("exec: log");
 #endif
 
 			if (status) {
-				switch (r -> data.log.priority) {
+				switch (r->data.log.priority) {
 				case log_priority_fatal:
 					log_fatal ("%.*s", (int)ds.len,
 						ds.data);
@@ -517,6 +531,14 @@ int execute_statements (result, packet, lease, client_state,
 
 			break;
 
+		      case vendor_opt_statement:
+		        /* If possible parse any options in a vendor option
+			 * encapsulation, this may add options to the in_options
+			 * option state */
+			parse_vendor_option(packet, lease, client_state,
+					    in_options, out_options, scope);
+			break;
+
 		      default:
 			log_error ("bogus statement type %d", r -> op);
 			break;
@@ -540,7 +562,7 @@ int execute_statements (result, packet, lease, client_state,
 
 void execute_statements_in_scope (result, packet,
 				  lease, client_state, in_options, out_options,
-				  scope, group, limiting_group)
+				  scope, group, limiting_group, on_star)
 	struct binding_value **result;
 	struct packet *packet;
 	struct lease *lease;
@@ -550,6 +572,7 @@ void execute_statements_in_scope (result, packet,
 	struct binding_scope **scope;
 	struct group *group;
 	struct group *limiting_group;
+	struct on_star *on_star;
 {
 	struct group *limit;
 
@@ -589,9 +612,10 @@ void execute_statements_in_scope (result, packet,
 		execute_statements_in_scope (result, packet,
 					     lease, client_state,
 					     in_options, out_options, scope,
-					     group -> next, limiting_group);
+					     group->next, limiting_group,
+					     on_star);
 	execute_statements (result, packet, lease, client_state, in_options,
-			    out_options, scope, group -> statements);
+			    out_options, scope, group->statements, on_star);
 }
 
 /* Dereference or free any subexpressions of a statement being freed. */
@@ -756,11 +780,11 @@ void write_statements (file, statements, indent)
 			}
 			if (r -> data.on.evtypes & ON_COMMIT) {
 				fprintf (file, "%scommit", s);
-				s = "or";
+				s = " or ";
 			}
 			if (r -> data.on.evtypes & ON_RELEASE) {
 				fprintf (file, "%srelease", s);
-				s = "or";
+				/* s = " or "; */
 			}
 			if (r -> data.on.statements) {
 				fprintf (file, " {");
@@ -840,7 +864,7 @@ void write_statements (file, statements, indent)
 		      case eval_statement:
 			indent_spaces (file, indent);
 			fprintf (file, "eval ");
-			col = write_expression (file, r -> data.eval,
+			(void) write_expression (file, r -> data.eval,
 						indent + 5, indent + 5, 1);
 			fprintf (file, ";");
 			break;
@@ -914,11 +938,11 @@ void write_statements (file, statements, indent)
 			fprintf (file, "set ");
 			col = token_print_indent (file, indent + 4, indent + 4,
 						  "", "", r -> data.set.name);
-			col = token_print_indent (file, col, indent + 4,
+			(void) token_print_indent (file, col, indent + 4,
 						  " ", " ", "=");
 			col = write_expression (file, r -> data.set.expr,
 						indent + 3, indent + 3, 0);
-			col = token_print_indent (file, col, indent + 4,
+			(void) token_print_indent (file, col, indent + 4,
 						  " ", "", ";");
 			break;
 			
@@ -927,7 +951,7 @@ void write_statements (file, statements, indent)
 			fprintf (file, "unset ");
 			col = token_print_indent (file, indent + 6, indent + 6,
 						  "", "", r -> data.set.name);
-			col = token_print_indent (file, col, indent + 6,
+			(void) token_print_indent (file, col, indent + 6,
 						  " ", "", ";");
 			break;
 
@@ -938,29 +962,29 @@ void write_statements (file, statements, indent)
 						  "", "", "(");
 			switch (r -> data.log.priority) {
 			case log_priority_fatal:
-				col = token_print_indent
+				(void) token_print_indent
 					(file, col, indent + 4, "",
 					 " ", "fatal,");
 				break;
 			case log_priority_error:
-				col = token_print_indent
+				(void) token_print_indent
 					(file, col, indent + 4, "",
 					 " ", "error,");
 				break;
 			case log_priority_debug:
-				col = token_print_indent
+				(void) token_print_indent
 					(file, col, indent + 4, "",
 					 " ", "debug,");
 				break;
 			case log_priority_info:
-				col = token_print_indent
+				(void) token_print_indent
 					(file, col, indent + 4, "",
 					 " ", "info,");
 				break;
 			}
 			col = write_expression (file, r -> data.log.expr,
 						indent + 4, indent + 4, 0);
-			col = token_print_indent (file, col, indent + 4,
+			(void) token_print_indent (file, col, indent + 4,
 						  "", "", ");");
 
 			break;
@@ -977,13 +1001,18 @@ void write_statements (file, statements, indent)
                         	col = token_print_indent(file, col, indent + 4, "", " ", ",");
                                 col = write_expression (file, expr->data.arg.val, col, indent + 4, 0);
                         }
-                        col = token_print_indent(file, col, indent + 4, "", "", ");");
+                        (void) token_print_indent(file, col, indent + 4, "", "", ");");
 #else /* !ENABLE_EXECUTE */
 		        log_fatal("Impossible case at %s:%d (ENABLE_EXECUTE "
                                   "is not defined).", MDL);
 #endif /* ENABLE_EXECUTE */
                         break;
 			
+		      case vendor_opt_statement:
+			indent_spaces (file, indent);
+			fprintf (file, "parse-vendor-option;");
+			break;
+
 		      default:
 			log_fatal ("bogus statement type %d\n", r -> op);
 		}
@@ -1023,14 +1052,14 @@ int find_matching_case (struct executable_statement **ep,
 				sub = (evaluate_data_expression
 				       (&cd, packet, lease, client_state,
 					in_options, out_options,
-					scope, s -> data.c_case, MDL));
+					scope, s->data.c_case, MDL));
 				if (sub && cd.len == ds.len &&
 				    !memcmp (cd.data, ds.data, cd.len))
 				{
 					data_string_forget (&cd, MDL);
 					data_string_forget (&ds, MDL);
 					executable_statement_reference
-						(ep, s -> next, MDL);
+						(ep, s->next, MDL);
 					return 1;
 				}
 				data_string_forget (&cd, MDL);
@@ -1046,15 +1075,15 @@ int find_matching_case (struct executable_statement **ep,
 						      scope, expr);
 
 		if (status) {
-		    for (s = stmt; s; s = s -> next) {
+		    for (s = stmt; s; s = s->next) {
 			if (s -> op == case_statement) {
 				sub = (evaluate_numeric_expression
 				       (&c, packet, lease, client_state,
 					in_options, out_options,
-					scope, s -> data.c_case));
+					scope, s->data.c_case));
 				if (sub && n == c) {
 					executable_statement_reference
-						(ep, s -> next, MDL);
+						(ep, s->next, MDL);
 					return 1;
 				}
 			}
@@ -1064,11 +1093,11 @@ int find_matching_case (struct executable_statement **ep,
 
 	/* If we didn't find a matching case statement, look for a default
 	   statement and return the statement following it. */
-	for (s = stmt; s; s = s -> next)
-		if (s -> op == default_statement)
+	for (s = stmt; s; s = s->next)
+		if (s->op == default_statement)
 			break;
 	if (s) {
-		executable_statement_reference (ep, s -> next, MDL);
+		executable_statement_reference (ep, s->next, MDL);
 		return 1;
 	}
 	return 0;
@@ -1083,17 +1112,17 @@ int executable_statement_foreach (struct executable_statement *stmt,
 	struct executable_statement *foo;
 	int ok = 0;
 
-	for (foo = stmt; foo; foo = foo -> next) {
+	for (foo = stmt; foo; foo = foo->next) {
 	    if ((*callback) (foo, vp, condp) != 0)
 		ok = 1;
-	    switch (foo -> op) {
+	    switch (foo->op) {
 	      case null_statement:
 		break;
 	      case if_statement:
-		if (executable_statement_foreach (foo -> data.ie.tc,
+		if (executable_statement_foreach (foo->data.ie.tc,
 						  callback, vp, 1))
 			ok = 1;
-		if (executable_statement_foreach (foo -> data.ie.fc,
+		if (executable_statement_foreach (foo->data.ie.fc,
 						  callback, vp, 1))
 			ok = 1;
 		break;
@@ -1115,17 +1144,17 @@ int executable_statement_foreach (struct executable_statement *stmt,
 		break;
 	      case statements_statement:
 		if ((executable_statement_foreach
-		     (foo -> data.statements, callback, vp, condp)))
+		     (foo->data.statements, callback, vp, condp)))
 			ok = 1;
 		break;
 	      case on_statement:
 		if ((executable_statement_foreach
-		     (foo -> data.on.statements, callback, vp, 1)))
+		     (foo->data.on.statements, callback, vp, 1)))
 			ok = 1;
 		break;
 	      case switch_statement:
 		if ((executable_statement_foreach
-		     (foo -> data.s_switch.statements, callback, vp, 1)))
+		     (foo->data.s_switch.statements, callback, vp, 1)))
 			ok = 1;
 		break;
 	      case case_statement:
@@ -1138,14 +1167,15 @@ int executable_statement_foreach (struct executable_statement *stmt,
 		break;
 	      case let_statement:
 		if ((executable_statement_foreach
-		     (foo -> data.let.statements, callback, vp, 0)))
+		     (foo->data.let.statements, callback, vp, 0)))
 			ok = 1;
 		break;
 	      case define_statement:
 		break;
 	      case log_statement:
 	      case return_statement:
-              case execute_statement:
+	      case execute_statement:
+	      case vendor_opt_statement:
 		break;
 	    }
 	}
